package com.ea.easp;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.app.Activity;
import com.ea.easp.Debug;

#if defined INCLUDE_SOCIAL || defined INCLUDE_ORIGIN
import com.ea.easp.facebook.FacebookAgentJNI;
#endif

#if defined INCLUDE_MTX
import com.ea.easp.mtx.market.MarketJNI;
#endif

#if defined INCLUDE_STOREUI || defined INCLUDE_DMG || defined INCLUDE_ORIGIN
import com.ea.InAppWebBrowser.BrowserAndroid;
#endif

import android.view.ViewGroup;
import android.opengl.GLSurfaceView;

// Keyboard
import android.view.KeyEvent;
import android.content.res.Configuration;

public class EASPHandler
{
    private static final String kMODULE_TAG = "EASPHandler";

    public static Activity mActivity;
#if defined INCLUDE_STOREUI || defined INCLUDE_DMG || defined INCLUDE_ORIGIN
    public static ViewGroup mViewGroup;
#endif
    private GLSurfaceView mGLSurfaceView; // used to execute code in GL thread.
    private Handler mHandler;             // used to execute code in main(UI) thread.
    private TaskLauncher mTaskLauncher;   // used to execute code in GL and in UI thread.

#if defined INCLUDE_SOCIAL || defined INCLUDE_ORIGIN    
    private FacebookAgentJNI mFacebookAgent; // Handles Facebook login/logout procedure.
#endif

#if defined INCLUDE_MTX
    private MarketJNI mAndroidMarketJNI;     // Handles purchases/restores in Android.
#endif

#if defined INCLUDE_ORIGIN    
    private PhysicalKeyboardAndroid mPhysicalKeyboard;
#endif

    public native void initJNI();
    public native void shutdownJNI();

    public EASPHandler(Activity activity, ViewGroup viewGroup, GLSurfaceView glSurfaceView) {
        mActivity = activity;
#if defined INCLUDE_STOREUI || defined INCLUDE_DMG || defined INCLUDE_ORIGIN        
        mViewGroup = viewGroup;
#endif
        mGLSurfaceView = glSurfaceView;
        mHandler = new Handler();
        mTaskLauncher = new TaskLauncher(mHandler, mGLSurfaceView);
    }

    public void onCreate() {
        Debug.Log.d(kMODULE_TAG, "onCreate()...");

        initJNI();
        DeviceInfoUtil.init();
        PackageUtil.init();

#if defined INCLUDE_SOCIAL || defined INCLUDE_ORIGIN    
        // Facebook utils init.
        mFacebookAgent = new FacebookAgentJNI(mActivity, mTaskLauncher);
        mFacebookAgent.init();
#endif

#if defined INCLUDE_STOREUI || defined INCLUDE_DMG || defined INCLUDE_ORIGIN
        // DMG & StoreUI & Origin module init.
#if USE_EAMT_VERSION == 102
        BrowserAndroid.Init(mActivity, mViewGroup);
#else
        BrowserAndroid.Startup(mActivity, mViewGroup);
#endif
#endif

#if defined INCLUDE_MTX
        // MTX module init.
        mAndroidMarketJNI = new MarketJNI(mActivity, mTaskLauncher, MarketJNI.StoreType.UNKNOWN);
        mAndroidMarketJNI.init();
#endif
        
#if defined INCLUDE_ORIGIN    
        mPhysicalKeyboard = new PhysicalKeyboardAndroid(mTaskLauncher);
#endif
        Debug.Log.d(kMODULE_TAG, "...onCreate()");
    }

    public void onDestroy() {
        Debug.Log.d(kMODULE_TAG, "onDestroy()...");
        
#if defined INCLUDE_ORIGIN    
        mPhysicalKeyboard = null;
#endif 
        
#if defined INCLUDE_MTX
        mAndroidMarketJNI.shutdown();
#endif

#if defined INCLUDE_STOREUI || defined INCLUDE_DMG || defined INCLUDE_ORIGIN
        BrowserAndroid.Shutdown();
#endif

#if defined INCLUDE_SOCIAL || defined INCLUDE_ORIGIN    
        mFacebookAgent.shutdown();
#endif

        PackageUtil.shutdown();
        DeviceInfoUtil.shutdown();
        shutdownJNI();

        mActivity = null;
        Debug.Log.d(kMODULE_TAG, "...onDestroy()");
    }

    // called from native in GL thread.
    public void setLogEnabled(boolean enabled) {
        Debug.LogEnabled = enabled;
    }    
    
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
        Debug.Log.i(kMODULE_TAG, "onActivityResult requestCode=" + requestCode + ", resultCode=" + resultCode);
#if defined INCLUDE_SOCIAL || defined INCLUDE_ORIGIN    
        // here we can call handlers of activity's results.
        mFacebookAgent.authorizeCallback(requestCode, resultCode, intent);
#endif
    }

    public void onConfigurationChanged(Configuration newConfig) {
#if defined INCLUDE_ORIGIN    
        mPhysicalKeyboard.onConfigurationChanged(newConfig);
#endif
    }
    
    public boolean onKeyDown(int keyCode, KeyEvent event) {
#if defined INCLUDE_ORIGIN    
        return mPhysicalKeyboard.OnKeyDown(keyCode, event);
#else
        return false;
#endif
    }

    public boolean onKeyUp(int keyCode, KeyEvent event) {
#if defined INCLUDE_ORIGIN    
        return mPhysicalKeyboard.OnKeyUp(keyCode, event);
#else
        return false;
#endif
    }
}
